//ps2x68k keyboard converter by Blur2040 with help from a friend who is much better at this. Shameless use of libraries to make things easier.  

/*  PS2Keyboard library example
  
  PS2Keyboard now requries both pins specified for begin()

  keyboard.begin(data_pin, irq_pin);
  
  Valid irq pins:
     Arduino Uno:  2, 3
     Arduino Due:  All pins, except 13 (LED)
     Arduino Mega: 2, 3, 18, 19, 20, 21
     Teensy 2.0:   All pins, except 13 (LED)
     Teensy 2.0:   5, 6, 7, 8
     Teensy 1.0:   0, 1, 2, 3, 4, 6, 7, 16
     Teensy++ 2.0: 0, 1, 2, 3, 18, 19, 36, 37
     Teensy++ 1.0: 0, 1, 2, 3, 18, 19, 36, 37
     Sanguino:     2, 10, 11
  
  for more information you can read the original wiki in arduino.cc
  at http://www.arduino.cc/playground/Main/PS2Keyboard
  or http://www.pjrc.com/teensy/td_libs_PS2Keyboard.html
  
  Like the Original library and example this is under LGPL license.
  
  Modified by Cuninganreset@gmail.com on 2010-03-22
  Modified by Paul Stoffregen <paul@pjrc.com> June 2010
*/
   
#include <PS2Keyboard.h>
#include <SoftwareSerial.h>

SoftwareSerial mySerial(10, 11); // RX, TX

const int DataPin = 8;
const int IRQpin =  3;
const uint8_t keymap_at_tab[] = {
        0x00,	// 00h []
	0x6B,	// 01h [F9]
	0x00,	// 02h []
	0x67,	// 03h [F5]
	0x65,	// 04h [F3]
	0x63,	// 05h [F1]
	0x64,	// 06h [F2]
	0x00,	// 07h [F12]
	0x3b,	// 08h [leftarrow]
	0x6c,	// 09h [F10]
	0x3e,	// 0Ah [downarrow]
	0x3c,	// 0Bh [uparrow]
	0x66,	// 0Ch [F4]
	0x1d,	// 0Dh [return]
	0x61,	// 0Eh [`]
	0x00,	// 0Fh []
	0x00,	// 10h []
	0x73,	// 11h [LEFT ALT]
	0x70,	// 12h [LEFT SHIFT]
	0x00,	// 13h []
	0x71,	// 14h [LEFT CTRL]
	0x3d,	// 15h [rightarrow]
	0x02,	// 16h [1]
	0x00,	// 17h []
	0x00,	// 18h []
	0x00,	// 19h []
	0x2a,	// 1Ah [Z]
	0x1f,	// 1Bh [S]
	0x1e,	// 1Ch [A]
	0x12,	// 1Dh [W]
	0x03,	// 1Eh [2]
	0x00,	// 1Fh []
	0x35,	// 20h [space]
	0x2c,	// 21h [C]
	0x2b,	// 22h [X]
	0x20,	// 23h [D]
	0x13,	// 24h [E]
	0x05,	// 25h [4]
	0x04,	// 26h [3]
	0x00,	// 27h []
	0x00,	// 28h []
	0x35,	// 29h [SPACE]
	0x2d,	// 2Ah [V]
	0x21,	// 2Bh [F]
	0x15,	// 2Ch [T]
	0x14,	// 2Dh [R]
	0x06,	// 2Eh [5]
	0x00,	// 2Fh []
	0x0b,	// 30h [0]
	0x02,	// 31h [1]
	0x03,	// 32h [2]
	0x04,	// 33h [3]
	0x05,	// 34h [4]
	0x06,	// 35h [5]
	0x07,	// 36h [6]
	0x08,	// 37h [7]
	0x09,	// 38h [8]
	0x0a,	// 39h [9]
	0x30,	// 3Ah [M]
	0x24,	// 3Bh [J]
	0x17,	// 3Ch [U]
	0x08,	// 3Dh [7]
	0x09,	// 3Eh [8]
	0x00,	// 3Fh []
	0x00,	// 40h []
	0x1e,	// 41h [A]
	0x2e,	// 42h [B]
	0x2c,	// 43h [C]
	0x20,	// 44h [D]
	0x13,	// 45h [E]
	0x21,	// 46h [F]
	0x22,	// 47h [G]
	0x24,	// 48h [H]
	0x28,	// 49h [I]
	0x24,	// 4Ah [J]
	0x25,	// 4Bh [K]
	0x26,	// 4Ch [L]
	0x30,	// 4Dh [M]
	0x2f,	// 4Eh [N]
	0x19,	// 4Fh [O]
	0x1a,	// 50h [P]
	0x11,	// 51h [Q]
	0x14,	// 52h [R]
	0x1f,	// 53h [S]
	0x15,	// 54h [T]
	0x17,	// 55h [U]
	0x2d,	// 56h [V]
	0x12,	// 57h [W]
	0x2b,	// 58h [X]
	0x16,	// 59h [Y]
	0x2a,	// 5Ah [Z]
	0x29,	// 5Bh []]
	0x00,	// 5Ch []
	0x0e,	// 5Dh [\]
	0x00,	// 5Eh []
	0x00,	// 5Fh []
	0x00,	// 60h []
	0x1e,	// 61h [a]
	0x2e,	// 62h [b]
	0x2c,	// 63h [c]
	0x20,	// 64h [d]
	0x13,	// 65h [e]
	0x21,	// 66h [f]
	0x22,	// 67h [g]
	0x23,	// 68h [h]
	0x18,	// 69h [i]
	0x24,	// 6Ah [j]
	0x25,	// 6Bh [k]
	0x26,	// 6Ch [l]
	0x30,	// 6Dh [m]
	0x2f,	// 6Eh [n]
	0x19,	// 6Fh [o]
	0x1a,	// 70h [p]
	0x11,	// 71h [q]
	0x14,	// 72h [r]
	0x1f,	// 73h [s]
	0x15,	// 74h [t]
	0x17,	// 75h [u]
	0x2d,	// 76h [v]
	0x12,	// 77h [w]
	0x2b,	// 78h [x]
	0x16,	// 79h [y]
	0x2a,	// 7Ah [z]
	0x42,	// 7Bh [KEYPAD -]
	0x41,	// 7Ch [KEYPAD *]
	0x45,	// 7Dh [KEYPAD 9]
	0x00,	// 7Eh [SCROLL LOCK]
	0x0f,	// 7Fh [backspace]
	0x63,	// 80h [f1]
	0x64,	// 81h [f2]
	0x65,	// 82h [f3]
	0x66,	// 83h [f4]
	0x67,	// 84h [f5]
	0x68,	// 85h [f6]
	0x69,	// 86h [f7]
	0x6a,	// 87h [f8]
	0x6b,	// 88h [f9]
	0x6c,	// 89h [f10]
        0x6d,	// 8Ah [f11]
	0x6e	// 8Bh [f12]
};

const uint8_t keymap_mod_tab[] = {

	0x00,	// 00h []
	0x00,	// 01h []
	0x00,	// 02h []
	0x00,	// 03h []
	0x00,	// 04h []
	0x00,	// 05h []
	0x00,	// 06h []
	0x00,	// 07h []
	0x00,	// 08h []
	0x00,	// 09h []
	0x00,	// 0Ah []
	0x00,	// 0Bh []
	0x00,	// 0Ch []
	0x00,	// 0Dh []
	0x00,	// 0Eh []
	0x00,	// 0Fh []
	0x00,	// 10h [WWW SEARCH]
	0x00,	// 11h [RIGHT APP]
	0x00,	// 12h []
	0x00,	// 13h []
	0x00,	// 14h [RIGHT CTRL]
	0x00,	// 15h [PREVIOUS TRACK]
	0x00,	// 16h []
	0x00,	// 17h []
	0x00,	// 18h [WWW FAVORITES]
	0x00,	// 19h []
	0x00,	// 1Ah []
	0x00,	// 1Bh []
	0x00,	// 1Ch []
	0x00,	// 1Dh []
	0x00,	// 1Eh []
	0x55,	// 1Fh [LEFT MOD]
	0x00,	// 20h [WWW REFRESH]
	0x00,	// 21h [VOL DOWN]
	0x00,	// 22h []
	0x00,	// 23h [MUTE]
	0x00,	// 24h []
	0x00,	// 25h []
	0x00,	// 26h []
	0x00,	// 27h [RIGHT MOD]
	0x00,	// 28h [WWW STOP]
	0x00,	// 29h []
	0x00,	// 2Ah []
	0x00,	// 2Bh [CALCULATOR]
	0x00,	// 2Ch []
	0x00,	// 2Dh []
	0x00,	// 2Eh []
	0x00,	// 2Fh [RIGHT APPS]
	0x00,	// 30h [WWW FORWARD]
	0x00,	// 31h []
	0x00,	// 32h [VOL UP]
	0x00,	// 33h []
	0x00,	// 34h [PLAY/PAUSE]
	0x00,	// 35h []
	0x00,	// 36h []
	0x00,	// 37h [POWER]
	0x00,	// 38h [WWW BACK]
	0x00,	// 39h []
	0x00,	// 3Ah [WWW HOME]
	0x00,	// 3Bh [STOP]
	0x00,	// 3Ch []
	0x00,	// 3Dh []
	0x00,	// 3Eh []
	0x00,	// 3Fh [SLEEP]
	0x00,	// 40h [MY COMPUTER]
	0x00,	// 41h []
	0x00,	// 42h []
	0x00,	// 43h []
	0x00,	// 44h []
	0x00,	// 45h []
	0x00,	// 46h []
	0x00,	// 47h []
	0x00,	// 48h [EMAIL]
	0x00,	// 49h []
	0x40,	// 4Ah [KEYPAD /]
	0x00,	// 4Bh []
	0x00,	// 4Ch []
	0x00,	// 4Dh [NEXT TRACK]
	0x00,	// 4Eh []
	0x00,	// 4Fh []
	0x00,	// 50h [MEDIA SELECT]
	0x00,	// 51h []
	0x00,	// 52h []
	0x00,	// 53h []
	0x00,	// 54h []
	0x00,	// 55h []
	0x00,	// 56h []
	0x00,	// 57h []
	0x00,	// 58h []
	0x00,	// 59h []
	0x4e,	// 5Ah [KEYPAD ENTER]
	0x00,	// 5Bh []
	0x00,	// 5Ch []
	0x00,	// 5Dh []
	0x00,	// 5Eh [WAKE]
	0x00,	// 5Fh []
	0x00,	// 60h []
	0x00,	// 61h []
	0x00,	// 62h []
	0x00,	// 63h []
	0x00,	// 64h []
	0x00,	// 65h []
	0x00,	// 66h []
	0x00,	// 67h []
	0x00,	// 68h []
	0x3a,	// 69h [END]
	0x00,	// 6Ah []
	0x3b,	// 6Bh [ARROW LEFT]
	0x36,	// 6Ch [HOME]
	0x00,	// 6Dh []
	0x00,	// 6Eh []
	0x00,	// 6Fh []
	0x5e,	// 70h [INSERT]
	0x37,	// 71h [DELETE]
	0x3e,	// 72h [ARROW DOWN]
	0x00,	// 73h []
	0x3d,	// 74h [ARROW RIGHT]
	0x3c,	// 75h [ARROW UP]
	0x00,	// 76h []
	0x00,	// 77h []
	0x00,	// 78h []
	0x00,	// 79h []
	0x39,	// 7Ah [PAGE DOWN]
	0x00,	// 7Bh []
	0x00,	// 7Ch []
	0x38,	// 7Dh [PAGE UP]
	0x00,	// 7Eh []
	0x00,	// 7Fh []
	0x63,	// 80h [f1]
	0x64,	// 81h [f2]
	0x65,	// 82h [f3]
	0x66,	// 83h [f4]
	0x67,	// 84h [f5]
	0x68,	// 85h [f6]
	0x69,	// 86h [f7]
	0x6a,	// 87h [f8]
	0x6b,	// 88h [f9]
	0x6c,	// 89h [f10]
        0x6d,	// 8Ah [f11]
	0x6e	// 8Bh [f12]
};

PS2Keyboard keyboard;

void setup() {
  
  delay(1000);
  keyboard.begin(DataPin, IRQpin);
  mySerial.begin(2400);
  //mySerial.println("Keyboard Test:");
  Serial.begin(57600);
  
}

void loop() {
  if (keyboard.available()) {
    
    // read the next key
    int c = keyboard.read();
    // check for some of the special keys
    
    /*if (c > 0x40 && c < 0x5b) {
      mySerial.write(0x70);
      mySerial.write(keymap_at_tab[c]);
      Serial.print(c);
    } else if (c == 33) {
      mySerial.write(44);
    } else if (c == 73) {
      mySerial.write(0x32);
    } else if (c == 224) {
      mySerial.write(62);
      delay(10);
      char sub = keyboard.readScanCode();
      mySerial.write(keymap_mod_tab[sub]);
    } else if (c == PS2_PAGEUP) {
      mySerial.print("[PgUp]");
    } else if (c == PS2_LEFTARROW) {
      mySerial.print("[Left]");
    } else if (c == PS2_RIGHTARROW) {
      mySerial.print("[Right]");
    } else if (c == PS2_UPARROW) {
      mySerial.print("[Up]");
    } else if (c == PS2_DOWNARROW) {
      mySerial.print("[Down]");
    } else if (c == PS2_DELETE) {
      mySerial.print("[Del]");
    } else if (c == 00) {
      mySerial.print("");
      Serial.print("");
    } else {*/
      
      // otherwise, just print all normal characters
       mySerial.write(keymap_at_tab[c]);
       Serial.print(c);
       Serial.print(",");
       Serial.print(keymap_at_tab[c]);
       Serial.print(",");
     //}
  }
}
